# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging
from collections import OrderedDict

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimList,
    HprimMultiple,
    HprimString,
    Segment,
)
from driverng.astm.parser import RecursiveList

logger = logging.getLogger("driverng.drivers.cobas_pvt")


class PvtSegmentH(HeaderSegment):
    sender_id = HprimString((5, 1))
    pvt_system_type = HprimString((5, 2))
    software_version = HprimString((5, 3))
    sender_adress = HprimString(6, default="MPL")
    receiver_id = HprimString(10)
    comment = HprimString(11, default="PVT")
    processing_id = HprimString(14, default="P")
    version = HprimString(15, necessary=True)

    patients = HprimForeign("P", "patients")
    queries = HprimForeign("Q", "queries")
    infos = HprimForeign("M", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class PvtSegmentP(Segment):
    patient_id = HprimString(3)
    lab_patient_id = HprimString(4)
    patient_id_no3 = HprimString(5)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    maiden_name = HprimString(7)
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)
    attending_physician_id = HprimString((14, 1))
    diagnosis = HprimString(19)
    admission_date = HprimDate(24)
    admission_status = HprimString(25, necessary=True)
    location = HprimString(26)

    orders = HprimForeign("O", "orders")


class PvtSegmentO(Segment):
    specimen_id = HprimString(3)
    pvt_speciment_id = HprimString(4, default="0")
    universal_test_id = HprimList(
        5,
        basefield=HprimMultiple(
            0,
            code=HprimString(4),
            pre_result=HprimString(5),
            date_result_created=HprimDate(6),
        ),
    )
    priority = HprimString(6)
    request_date = HprimDate(7)
    action_code = HprimString(12)
    danger_code = HprimString(13, necessary=True)
    date_specimen_received = HprimDate(15)
    specimen_descriptor = HprimString(16)
    # State of sample for sorting
    user_field_no2 = HprimString(20)
    report_type = HprimString(26)

    results = HprimForeign("R", "results")


class PvtSegmentQ(Segment):
    specimen_id = HprimString((3, 2))
    universal_test_id = HprimString(5)
    user_field_no1 = HprimString(11)
    user_field_no2 = HprimString(12)
    request_status_codes = HprimString(13)


class PvtSegmentR(Segment):
    test_code = HprimString((3, 4))
    value = HprimString(4)
    abnormal_flags = HprimList(7)
    result_status = HprimList(9)
    date_test_completed = HprimDate(13)
    pvt_system_identification = HprimString(14)


class PvtSegmentM(Segment):
    record_id = HprimString(3)


class PvtSegmentMLI(Segment):
    record_id = HprimString(3)
    subrack_name = HprimString((4, 1))
    virtual_workplace_name = HprimString((4, 2))
    block_number = HprimString((5, 1))
    position = HprimString((5, 2))
    sample_id = HprimString(6)
    test_id = HprimList(7)
    volume = HprimString(8)
    specimen_sorted_date = HprimDate(9)
    tube_type = HprimString(10)
    original_id = HprimString(11)
    error_code = HprimString(12)
    customer_name = HprimString(13)


class PvtSegmentMIS(Segment):
    record_id = HprimString(3)
    specimen_id = HprimString(4)
    tube_type = HprimString(5)
    quality = HprimString((6, 1))
    quality_class = HprimString((6, 2))
    clots = HprimString(7)
    volume = HprimString(8)
    lower_level_serum = HprimString(9)
    upper_level_serum = HprimString(10)
    error_code = HprimString(11)
    outside_diameter = HprimString(12)
    inside_diameter = HprimString(13)
    number_pictures = HprimString(14)


class PvtSegmentMTR(Segment):
    record_id = HprimString(3)
    workplace = HprimString(4)
    block_number = HprimString(5)
    tray_change_date = HprimDate(6)


class PvtSegmentMCY(Segment):
    # Segment existant mais non documenté dans le protocole
    record_id = HprimString(3)


class PvtSegmentMCC(Segment):
    # Segment existant mais non documenté dans le protocole
    record_id = HprimString(3)


class PvtSegmentMCP(Segment):
    # Segment existant mais non documenté dans le protocole
    record_id = HprimString(3)


class PvtSegmentMCT(Segment):
    # Segment existant mais non documenté dans le protocole
    record_id = HprimString(3)


class PvtSegmentL(Segment):
    terminaison_code = HprimString(3, default="N")
    empty_field = HprimString(4, necessary=True)


class Driver(AstmMessageV2):

    SPEC = {"code": "cobas_pvt", "label": "PVT trieur Cobas P612-P512-P671-P471"}

    SEGMENT_CLASS_MAP = {
        "H": PvtSegmentH,
        "P": PvtSegmentP,
        "O": PvtSegmentO,
        "Q": PvtSegmentQ,
        "R": PvtSegmentR,
        "M": PvtSegmentM,
        "L": PvtSegmentL,
    }

    SEGMENT_M = {
        "LI": PvtSegmentMLI,
        "IS": PvtSegmentMIS,
        "TR": PvtSegmentMTR,
        "CY": PvtSegmentMCY,
        "CC": PvtSegmentMCC,
        "CP": PvtSegmentMCP,
        "CT": PvtSegmentMCT,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = PvtSegmentH(type_segment="H", **kwargs)
        message.end = PvtSegmentL(type_segment="L")
        return message

    def parse_segment(self, line, separators):
        segment = super(Driver, self).parse_segment(line, separators)
        if segment.type_segment == "M":
            if segment.record_id in self.SEGMENT_M:
                seg_m = self.SEGMENT_M[segment.record_id]()
                splitted = RecursiveList.split(line, separators)
                seg_m.populate(splitted)
                return seg_m
            else:
                logger.info(
                    "Le record ID (champ M.3) %s du segment M n'est pas traité",
                    segment.record_id,
                )
        return segment

    def encode_worklist(self, worklist, context):
        message = self.create()
        for patient in worklist.get_astm_worklist(
            unique_together=["priority", "action"],
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
            action={"del": "C", "add": "A", "new": "N"},
        ):
            seg_p = message.patients.add(
                patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
            )
            for sample, priority, action in patient.samples:
                seg_p.patient_id_no3 = sample.sample_id
                seg_p.admission_date = sample.request_date
                seg_o = seg_p.orders.add(
                    specimen_id=sample.sample_id,
                    priority=priority,
                    action_code=action,
                    request_date=sample.request_date,
                )
                order_status = OrderedDict()
                for order in sample["orders"]:
                    order_status.setdefault(order.code, [])
                    order_status[order.code].append(order.dem_auto_test_status)
                logger.info(
                    "P612 sample_id %s order_status %s", sample.sample_id, order_status
                )
                archive = False
                logger.info(
                    "P612 sample_id %s orders_status %s",
                    sample.sample_id,
                    order_status.items(),
                )
                _, status_min = min(order_status.items(), key=lambda x: x[1][0])
                _, status_max = max(order_status.items(), key=lambda x: x[1][0])
                if min(status_min) in (3, 4, 6):
                    t = seg_o.universal_test_id.add()
                    t.code = "TARCHI"
                    t.pre_result = 0
                    archive = True
                else:
                    archive = False
                zone_urgence = ""
                # Il faut envoyer la zone urgence lorsque qu'il y a une
                # priorité et lorsque tous les status des tests de
                # l'échantillon sont au status 0 (tests demandés non encore
                # chargés sur l'automate)
                if priority == "S" and min(status_min) == 0 and max(status_max) == 0:
                    zone_urgence = "URG"
                # Les tests sont regroupés par zone de test, on peut avoir
                # plusieurs mêmes codes demandes correspondant à une zone
                # On retire les doublons de codes demandes dans la liste des
                # demandes de tests
                if not archive:
                    orders_codes_list = [
                        (order["code"], order["dem_auto_test_status"])
                        for order in sample.orders
                    ]
                    if zone_urgence:
                        orders_codes_list.append((zone_urgence, 0))
                    logger.info("P612 orders_codes_list %s", (orders_codes_list))
                    for order, status in sorted(list(set(orders_codes_list))):
                        if status in (3, 4, 6):
                            continue
                        t = seg_o.universal_test_id.add()
                        t.code = order
                        t.pre_result = 0
        if len(message.patients):
            return message
        elif context["query"] and context["query"].queries:
            for seg_q in context["query"].queries:
                seg_p = message.patients.add(
                    patient_id=seg_q.specimen_id, lab_patient_id=seg_q.specimen_id
                )
                seg_o = seg_p.orders.add(
                    specimen_id=seg_q.specimen_id,
                    priority="R",
                    action_code="N",
                    user_field_no2="X",
                    report_type="Z",
                )
            return message

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(type="patient", sample_id=seg_q.specimen_id)
        else:
            worklist = Worklist(context="result")
            for seg_m in message.infos:
                if seg_m.record_id == "LI":
                    if seg_m.subrack_name and seg_m.block_number and seg_m.position:
                        comments = []
                        if seg_m.virtual_workplace_name:
                            rack = "{}-{}-{}".format(
                                seg_m.virtual_workplace_name,
                                seg_m.block_number,
                                seg_m.position,
                            )
                        else:
                            rack = "{}-{}-{}".format(
                                seg_m.subrack_name, seg_m.block_number, seg_m.position
                            )
                        sample = worklist.get_or_create(
                            sample_id=seg_m.sample_id, rack=rack, type="patient"
                        )
                        if seg_m.volume:
                            comments.append(
                                "Volume {} microlitres".format(seg_m.volume)
                            )
                        sample.results.add(
                            code="TRACKINGP612",
                            value=seg_m.subrack_name,
                            comments=comments,
                            date=seg_m.specimen_sorted_date,
                            instrument=seg_m.subrack_name,
                        )
        return worklist
