# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv

from six import StringIO

from driverng import MessageV2, Worklist
from driverng.splitted import Multiple, Splitted, String


class NaviosBase(Splitted):
    SEPARATOR = "\t"


class NaviosResult(object):
    FIELDS = [
        String("file"),
        String("gate"),
        String("x_param"),
        String("y_param"),
        String("barcode"),
        String("region"),
        Multiple("results", (String("resu"),), separator="\t"),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for i, field in enumerate(fields):
            if isinstance(field, Multiple):
                setattr(self, field.name, line[i:])
                break
            setattr(self, field.name, line[i])


class NaviosMultiple(list):

    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: NaviosResult}

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        if data[-1] == "\t":
            data = data[:-1]
        buf = StringIO(data)
        reader = csv.reader(buf, delimiter=str("\t"))
        for line in reader:
            if not len(line):
                continue
            if len(line) in (8, 9):
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)


class Driver(MessageV2):
    SPEC = {"code": "navios", "label": "Navios FC50 Beckman"}
    OPTIONS = [
        MessageV2.Option(
            "navios_identification_qc",
            label='Identification des QC dans la colonne File séparés par des ";"',
            default="IMMUNOTROL",
        )
    ]

    def __init__(self, **kwargs):
        navios_identification_qc = kwargs.pop("navios_identification_qc", "IMMUNOTROL")
        if navios_identification_qc:
            self.identification_qc = navios_identification_qc.split(";")
        else:
            self.identification_qc = []
        super(Driver, self).__init__(**kwargs)

    def parse(self, data):
        obj = NaviosMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        RESULTS_CODES = ()
        for m in message:
            if m.file.strip() == "File" and m.gate.strip() == "Gate":
                RESULTS_CODES = m.results
                continue
            control = False
            for elt in self.identification_qc:
                if elt and m.file.find(elt) != -1:
                    control = True
                    break
            # Ne pas intégrer les résultats dont le nom du fichier présent dans
            # la colonne "File" ne débute pas par le code barre de la colonne
            # "Barcode"
            if not m.file.startswith(m.barcode.strip()):
                continue
            if control:
                sample = worklist.get_or_create(
                    control_lot=m.barcode.strip(),
                    sample_id=m.barcode.strip(),
                    type="control",
                )
            else:
                sample = worklist.get_or_create(
                    sample_id=m.barcode.strip(), type="patient"
                )
            for i, result in enumerate(m.results):
                if m.region.strip():
                    sample.results.add(
                        code="{}{}".format(m.region.strip(), RESULTS_CODES[i].strip()),
                        value=result.strip(),
                    )
        return worklist
